---
title: Demos
pagination_prev: getting-started/index
pagination_next: solutions/input
hide_table_of_contents: true
---

Demos include complete examples and short discussions.  For features that can
run in the web browser, demos will include interactive examples.

:::tip pass

If a demo for a library or framework is not included here, please leave a note
in the [issue tracker](https://git.sheetjs.com/sheetjs/docs.sheetjs.com/issues)

:::

### JavaScript APIs

- [`XMLHttpRequest and fetch`](/docs/demos/net/network)
- [`Clipboard Data`](/docs/demos/local/clipboard)
- [`Web Workers`](/docs/demos/bigdata/worker)
- [`Typed Arrays`](/docs/demos/math)
- [`Local File Access`](/docs/demos/local/file)
- [`LocalStorage and SessionStorage`](/docs/demos/local/storageapi)
- [`Web SQL Database`](/docs/demos/local/websql)
- [`IndexedDB`](/docs/demos/local/indexeddb)

### Web Frameworks

- [`Kaioken`](/docs/demos/frontend/kaioken)
- [`Angular`](/docs/demos/frontend/angular)
- [`ReactJS`](/docs/demos/frontend/react)
- [`Svelte`](/docs/demos/frontend/svelte)
- [`VueJS`](/docs/demos/frontend/vue)
- [`AngularJS`](/docs/demos/frontend/angularjs)
- [`Dojo`](/docs/demos/frontend/dojo)
- [`Knockout`](/docs/demos/frontend/legacy#knockoutjs)

### Front-End UI Components

- [`canvas-datagrid`](/docs/demos/grid/cdg)
- [`x-spreadsheet`](/docs/demos/grid/xs)
- [`react-data-grid`](/docs/demos/grid/rdg)
- [`glide-data-grid`](/docs/demos/grid/gdg)
- [`vue3-table-lite`](/docs/demos/grid/vtl)
- [`angular-ui-grid`](/docs/demos/grid/#angular-ui-grid)
- [`material ui`](/docs/demos/grid/mui)

### iOS and Android Mobile Apps

- [`React Native`](/docs/demos/mobile/reactnative)
- [`NativeScript`](/docs/demos/mobile/nativescript)
- [`Quasar`](/docs/demos/mobile/quasar)
- [`Ionic`](/docs/demos/mobile/ionic)
- [`CapacitorJS`](/docs/demos/mobile/capacitor)

### Desktop App Frameworks

- [`Electron`](/docs/demos/desktop/electron)
- [`NW.js`](/docs/demos/desktop/nwjs)
- [`Wails`](/docs/demos/desktop/wails)
- [`Tauri`](/docs/demos/desktop/tauri)
- [`NeutralinoJS`](/docs/demos/desktop/neutralino)
- [`React Native for Desktop`](/docs/demos/desktop/reactnative)

### Content Management and Static Sites

- [`Lume`](/docs/demos/static/lume)
- [`GatsbyJS`](/docs/demos/static/gatsbyjs)
- [`Eleventy`](/docs/demos/static/eleventy)
- [`ViteJS`](/docs/demos/static/vitejs)
- [`NextJS`](/docs/demos/static/nextjs)
- [`NuxtJS`](/docs/demos/static/nuxtjs)
- [`SvelteKit`](/docs/demos/static/svelte)
- [`Webpack`](/docs/demos/static/webpack)

### App Extensions

- [`Excel JavaScript API`](/docs/demos/extensions/excelapi)
- [`ExtendScript for Adobe Apps`](/docs/demos/extensions/extendscript)
- [`Chrome and Chromium Extensions`](/docs/demos/extensions/chromium)
- [`Google Sheets + Apps Script`](/docs/demos/extensions/gsheet)
- [`AppleScript and OSA`](/docs/demos/extensions/osa)
- [`Mathematica`](/docs/demos/extensions/mathematica)
- [`MATLAB`](/docs/demos/extensions/matlab)
- [`Stata`](/docs/demos/extensions/stata)
- [`Maple`](/docs/demos/extensions/maple)

### Cloud Platforms

- [`Amazon Web Services`](/docs/demos/cloud/aws)
- [`Azure Functions and Storage`](/docs/demos/cloud/azure)
- [`NetSuite SuiteScript`](/docs/demos/cloud/netsuite)
- [`Salesforce Lightning Web Components`](/docs/demos/cloud/salesforce)

### File Hosting Services

- [`Dropbox`](/docs/demos/cloud/dropbox)
- [`GitHub`](/docs/demos/cloud/github)

### Cloud Data Services

- [`Google Sheets`](/docs/demos/cloud/gsheet)
- [`Airtable`](/docs/demos/cloud/airtable)

### Platforms and Integrations

- [`Command-Line Tools`](/docs/demos/cli)
- [`NodeJS Server-Side Processing`](/docs/demos/net/server/#nodejs)
- [`Deno Server-Side Processing`](/docs/demos/net/server/#deno)
- [`Headless Automation`](/docs/demos/net/headless)
- [`Databases and Structured Data Stores`](/docs/demos/data)
- [`NoSQL and Unstructured Data Stores`](/docs/demos/data)
- [`Legacy Internet Explorer`](/docs/demos/frontend/legacy#internet-explorer)

### Bundlers and Tooling

- [`browserify`](/docs/demos/frontend/bundler/browserify)
- [`bun`]((/docs/getting-started/installation/bun#bundling)
- [`esbuild`](/docs/demos/frontend/bundler/esbuild)
- [`parcel`](/docs/demos/frontend/bundler/parcel)
- [`requirejs`](/docs/demos/frontend/bundler/requirejs)
- [`rollup`](/docs/demos/frontend/bundler/rollup)
- [`swc`](/docs/demos/frontend/bundler/swcpack)
- [`systemjs`](/docs/demos/frontend/bundler/systemjs)
- [`vite`](/docs/demos/frontend/bundler/vitejs)
- [`webpack`](/docs/demos/frontend/bundler/webpack)
- [`wmr`](/docs/demos/frontend/bundler/#wmr)

### Other Programming Languages

- [`JavaScript Engines`](/docs/demos/engines)
- [`V8 (C++ / Rust)`](/docs/demos/engines/v8)
- [`Duktape (C / Perl / Python / Zig)`](/docs/demos/engines/duktape)
- [`JavaScriptCore (Swift)`](/docs/demos/engines/jsc)
- [`Rhino (Java)`](/docs/demos/engines/rhino)
- [`Nashorn (Java)`](/docs/demos/engines/nashorn)
- [`Goja (Go)`](/docs/demos/engines/goja)
- [`ChakraCore (C++)`](/docs/demos/engines/chakra)
- [`QuickJS (C)`](/docs/demos/engines/quickjs)
- [`Jint (C# / .NET)`](/docs/demos/engines/jint)
- [`ExecJS (Ruby)`](/docs/demos/engines/rb)
- [`JavaScript::Engine (Perl)`](/docs/demos/engines/perl)
