#!/bin/bash
# https://docs.sheetjs.com/docs/demos/frontend/bundler/browserify
# requires global puppeteer and express

# sudo npm i -g puppeteer express@4
cd /tmp
rm -rf sheetjs-browserify-tests
mkdir sheetjs-browserify-tests
cd sheetjs-browserify-tests

for n in {3..17}; do
mkdir sheetjs-browserify-$n
cd sheetjs-browserify-$n
npm init -y

npm i --save https://cdn.sheetjs.com/xlsx-0.20.2/xlsx-0.20.2.tgz

cat >index.js <<EOF
const { utils, version, writeFileXLSX } = require('xlsx');

document.getElementById("xport").addEventListener("click", function() {
  /* fetch JSON data and parse */
  var url = "https://sheetjs.com/data/executive.json";
  fetch(url).then(function(res) { return res.json(); }).then(function(raw_data) {

    /* filter for the Presidents */
    var prez = raw_data.filter(function(row) { return row.terms.some(function(term) { return term.type === "prez"; }); });

    /* sort by first presidential term */
    prez.forEach(function(row) {
      row.start = row.terms.find(function(term) {
        return term.type === "prez";
      }).start
    });
    prez.sort(function(l,r) { return l.start.localeCompare(r.start); });

    /* flatten objects */
    var rows = prez.map(function(row) { return {
      name: row.name.first + " " + row.name.last,
      birthday: row.bio.birthday
    }; });

    /* generate worksheet and workbook */
    var worksheet = utils.json_to_sheet(rows);
    var workbook = utils.book_new();
    utils.book_append_sheet(workbook, worksheet, "Dates");

    /* fix headers */
    utils.sheet_add_aoa(worksheet, [["Name", "Birthday"]], { origin: "A1" });

    /* calculate column width */
    var max_width = rows.reduce(function(w, r) { return Math.max(w, r.name.length); }, 10);
    worksheet["!cols"] = [ { wch: max_width } ];

    /* create an XLSX file and try to save to Presidents.xlsx */
    //writeFileXLSX(workbook, "Presidents.xlsx");
    console.log(utils.sheet_to_csv(worksheet));
  });
});
EOF

npm install --save browserify@$n
npm ls | grep browserify
npx browserify@$n index.js > index.min.js

cat >index.html <<EOF
<!DOCTYPE html>
<html lang="en">
  <head></head>
  <body>
    <h1>SheetJS Presidents Demo</h1>
    <button id="xport">Click here to export</button>
    <script src="./index.min.js"></script>
  </body>
</html>
EOF

cat >test.js <<EOF
const puppeteer = require('puppeteer');
const express = require('express');
const app = express();
app.use(express.static('./'));
app.listen(7262, async() => {
	await new Promise((res,rej) => setTimeout(res, 1000));
  const browser = await puppeteer.launch();
  const page = await browser.newPage();
  page.on("console", msg => console.log("PAGE LOG:", msg.text()));
  await page.setViewport({width: 1920, height: 1080});
  await page.goto('http://localhost:7262/');
  await page.click("#xport");
	await new Promise((res,rej) => setTimeout(res, 1000));
  await browser.close();
	process.exit();
});
EOF

node test.js

cd -
done
